// TStopheShape
// V1.00  - 08.10.99 Delphi 3 (and higher ?)
// (c) Stophe, 1999.
// E-mail: gummi@chez.com
// FREEWARE : please just send me a e-mail if you use it ;-)
// NO WARRANTY

// Also, if you add features to TStopheShape, please, let me know. I'll be
// grateful to anybody will help me make TStopheShape bug-free, powerful
// and still available for free.

//Thanks for :
//    * gradient code : Gradient Panel V1.01,
//    * rotation text code : TLabelEffect V2.1.

unit StopheShape;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

{$R StopheShape.dcr}

const
  TAngleRange: array[1..3] of Integer = (0, 90, 270);
  NbCouleursMax = 255;

type
  TAlignement = (aLeft, aCenter, aRight);
  TDirection = (dBottomToTop, dHorizontal, dTopToBottom);

  TStopheShape = class(TGraphicControl)
  private
    { Private declarations }
    FCaption: String;
    FDirection: TDirection;
    FBeginColor, FEndColor: TColor;
    FTransparent: Boolean;
    FAlignement: TAlignement;
    FNumberOfColors: Integer;
    Colors : array[0..NbCouleursMax] of TColor;
    DDegToRad, DCosAngle, DSinAngle, DCosSquared, DSinSquared: Double;
    procedure SetCaption(Chaine: String);
    procedure SetAlignement(Value: TAlignement);
    procedure SetDirection(Value: TDirection);
    procedure SetTextAngle(Cnv: TCanvas; dDirection: TDirection);
    procedure SetNumberOfColors(Value: Integer);
    procedure SetBcolor(Value: TColor);
    procedure SetEcolor(Value: TColor);
    procedure SetTransparent(Value: Boolean);
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    procedure GradientFill(Canvas: TCanvas; Longueur, Hauteur: Integer);
    procedure CalColor;
  published
    { Published declarations }
    property BeginColor: Tcolor read FBeginColor write SetBcolor;
    property EndColor: Tcolor read FEndColor write SetEcolor;
    property NumberOfColors: Integer read FNumberOfColors write SetNumberOfColors;
    property Direction: TDirection read FDirection write SetDirection;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Caption: String read FCaption write SetCaption;
    property Alignement: TAlignement read FAlignement write SetAlignement;
    property Font;
    property Align;
    property Hint;
    property ShowHint;
    property ParentShowHint;
    property Visible;
		property OnClick;
		property OnDblClick;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Stophe VCL', [TStopheShape]);
end;

constructor TStopheShape.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCaption := 'Stophe';
  FBeginColor := clTeal;
  FEndColor := clInfoBk;
  FNumberOfColors := NbCouleursMax;
  FAlignement := aLeft;
  FDirection := dHorizontal;
  FTransparent := False;

  DDegToRad := PI / 180;
  DCosAngle := 1;
  DCosSquared := 1;
  DSinAngle := 0;
  DSinSquared := 0;

  Height := 30;
  Width := 100;
  with Font do begin
    Color := clBlack;
    Name := 'Arial';
    Size := 10;
    Style := [fsBold];
  end;
end;

procedure TStopheShape.SetTransparent(Value: Boolean);
begin
  if Value <> FTransparent then begin
    FTransparent := Value;
    Invalidate;
  end;
end;

procedure TStopheShape.SetCaption(Chaine: String);
begin
  Chaine := Trim(Chaine);
  if Chaine <> FCaption then begin
    FCaption := Chaine;
    Invalidate;
  end;
end;

procedure TStopheShape.SetAlignement(Value: TAlignement);
begin
  if Value <> FAlignement then begin
    FAlignement := Value;
    Invalidate;
  end;
end;

procedure TStopheShape.SetNumberOfColors(Value: Integer);
begin
  if FNumberOfColors <> Value then begin
    if Value > NbCouleursMax then
      FNumberOfColors := NbCouleursMax
    else
      FNumberOfColors := Value;
    Invalidate;
  end;
end;

procedure TStopheShape.SetBcolor(Value: TColor);
begin
  if FBeginColor <> value then begin
    FBeginColor := Value;
    Invalidate;
  end;
end;

procedure TStopheShape.SetEcolor(Value: TColor);
begin
  if FEndColor <> Value then begin
     FEndColor := Value;
     Invalidate;
  end;
end;

procedure TStopheShape.CalColor;
var
  BeginRGB: array[0..2] of Byte;
  RGBDifference: array[0..2] of Integer;
  i: Integer;
  R, G, B: Byte;
begin
  BeginRGB[0] := GetRValue(ColorToRGB(FBeginColor));
  BeginRGB[1] := GetGValue(ColorToRGB(FBeginColor));
  BeginRGB[2] := GetBValue(ColorToRGB(FBeginColor));

  RGBDifference[0] := GetRValue(ColorToRGB(FEndColor)) - BeginRGB[0];
  RGBDifference[1] := GetGValue(ColorToRGB(FEndColor)) - BeginRGB[1];
  RGBDifference[2] := GetBValue(ColorToRGB(FEndColor)) - BeginRGB[2];

  for i := 0 to NbCouleursMax do begin
    R := BeginRGB[0] + MulDiv(i, RGBDifference[0], FNumberOfColors - 1);
    G := BeginRGB[1] + MulDiv(i, RGBDifference[1], FNumberOfColors - 1);
    B := BeginRGB[2] + MulDiv(i, RGBDifference[2], FNumberOfColors - 1);
    Colors[i] := RGB(R, G, B);
  end;
end;

procedure TStopheShape.GradientFill(Canvas: TCanvas; Longueur, Hauteur: Integer);
var
  Rc: TRect;
  i: Integer;
  TmpBitmap: TBitmap;
begin
  CalColor;

  TmpBitmap := TBitmap.Create;
  with TmpBitmap do begin
    Width := Longueur;
    Height := Hauteur;
  end;
  case FDirection of
    dHorizontal:
    begin
      Rc.Top := 0;
      Rc.Bottom := Hauteur;
      with TmpBitmap.Canvas do begin
        for i := 0 to FNumberOfColors do begin
          Rc.Left := MulDiv(i, Longueur, FNumberOfColors);
          Rc.Right := MulDiv(i + 1, Longueur, FNumberOfColors);
          Brush.Color := Colors[i];
          FillRect(Rc);
        end;
      end;
    end;
    dBottomToTop, dTopToBottom:
    begin
      Rc.Left := 0;
      Rc.Right := TmpBitmap.Width;
      with TmpBitmap.Canvas do begin
        for i := 0 to FNumberOfColors do begin
          Rc.Bottom := MulDiv(i, Hauteur, FNumberOfColors);
          Rc.Top := MulDiv(i + 1, Hauteur, FNumberOfColors);
          if Direction = dBottomToTop then
            Brush.Color := Colors[FNumberOfColors - i - 1]
          else
            Brush.Color := Colors[i];

          FillRect(Rc);
        end;
      end;
    end;
  end;
  Canvas.Draw(0, 0, TmpBitmap);

  TmpBitmap.Free;
end;

procedure TStopheShape.SetDirection(Value: TDirection);
var
  Angle: Integer;
begin
  if Value <> FDirection then begin
    FDirection := Value;
    case FDirection of
      dTopToBottom:
        Angle := 90;
      dBottomToTop:
        Angle := 270;
      else
        Angle := 0;
    end;
    DCosAngle := Cos(Angle * DDegToRad);
    DCosSquared := DCosAngle * DCosAngle;
    DSinAngle := Sin(Angle * DDegToRad);
    DSinSquared := DSinAngle * DSinAngle;
    Invalidate;
  end;
end;

procedure TStopheShape.SetTextAngle(Cnv: TCanvas; dDirection: TDirection);
var
  FntLogRec: TLogFont;
  Angle: Integer;
begin
  case dDirection of
    dTopToBottom:
      Angle := 270;
    dBottomToTop:
      Angle := 90;
    else
      Angle := 0;
  end;
  GetObject(Cnv.Font.Handle, SizeOf(FntLogRec), Addr(FntLogRec));
  FntLogRec.lfEscapement := Angle * 10;
  FntLogRec.lfOutPrecision := OUT_TT_ONLY_PRECIS;
  Cnv.Font.Handle := CreateFontIndirect(FntLogRec);
end;

procedure TStopheShape.Paint;
var
  BmpWork: TBitmap;
begin
  BmpWork := TBitmap.Create;
  try
    with BmpWork do begin
      Height := Self.Height;
      Width := Self.Width;
      Canvas.Font := Self.Font;
      Canvas.CopyRect(Self.ClientRect, Self.Canvas, Self.ClientRect);
      if Direction <> dHorizontal then
        SetTextAngle(Canvas, FDirection);

      if not FTransparent then
        GradientFill(Canvas, Width, Height);

      Canvas.Brush.Style := bsClear;
      case FDirection of
        dHorizontal:
        begin
          case FAlignement of
          aLeft:
            Canvas.TextOut(5, (Height - Canvas.TextHeight(Caption)) div 2, Caption);
          aRight:
            Canvas.TextOut(Width - Canvas.TextWidth(Caption) - 5, (Height - Canvas.TextHeight(Caption)) div 2, Caption);
          aCenter:
            Canvas.TextOut((Width - Canvas.TextWidth(Caption)) div 2, (Height - Canvas.TextHeight(Caption)) div 2, Caption);
          end;
        end;
        dTopToBottom:
        begin
          case FAlignement of
          aLeft:
            Canvas.TextOut((Width + Canvas.TextHeight(Caption)) div 2, 5, Caption);
          aRight:
            Canvas.TextOut((Width + Canvas.TextHeight(Caption)) div 2, Height - Canvas.TextWidth(Caption) - 5, Caption);
          aCenter:
            Canvas.TextOut((Width + Canvas.TextHeight(Caption)) div 2, (Height - Canvas.TextWidth(Caption)) div 2, Caption);
          end;
        end;
        dBottomToTop:
        begin
          case FAlignement of
          aLeft:
            Canvas.TextOut((Width - Canvas.TextHeight(Caption)) div 2, Height - 5, Caption);
          aRight:
            Canvas.TextOut((Width - Canvas.TextHeight(Caption)) div 2, Canvas.TextWidth(Caption) + 5, Caption);
          aCenter:
            Canvas.TextOut((Width - Canvas.TextHeight(Caption)) div 2, (Height + Canvas.TextWidth(Caption)) div 2, Caption);
          end;
        end;
      end;

      Self.Canvas.CopyRect(Self.ClientRect, Canvas, Self.ClientRect);
    end;
  finally
    BmpWork.Free;
  end;
end;

end.
